#define CUSTOM_XPZ_EVENT_LOCK 90001
#define CUSTOM_XPZ_EVENT_UNLOCK 90002
#define CUSTOM_XPZ_EVENT_BIND 90003
#define CUSTOM_XPZ_EVENT_UNBIND 90004
#define CUSTOM_XPZ_EVENT_PARACHUTE 90005
#define CUSTOM_XPZ_EVENT_FREE 90006

extern const char *CFG;
extern const char *MAP_CFG;
extern const char *MOD_CFG;
extern const char *XPZ_VER;
extern const char *XPZ_BUILD;

extern INIClass *ini_main;
extern INIClass *ini_maps;
extern INIClass *ini_mods;

void XpzElite_Init();
unsigned __int64 iGetLevelExp(int iLevel);
const char *WideCharToChar(const wchar_t *wcs);
void ConsoleU(const char *param);
void ConsoleI(const char *param);
void Chat(int id,int type,const wchar_t *Message);
void Host(int PlayerID,int type,const char *Message);
void LoadLevel();
void GameOver();
void PJoin(int id, const char *name);
void Player_Leave(int id, bool new_map = false);
void BHS_Hook(int PlayerID,float ver);
bool file_exists(const char *file_name);

namespace Game {
	void ConsoleU(const char *param); // Console Output
	void ConsoleI(const char *param); // Console Input
	int Validate_Tank(const char *preset); // Gets the tanks level
	int Validate_Soldier(const char *preset); // Gets the character level
	int Get_Price(const char *preset); // Gets the price of a tank/character
	void New_Player(int ID, const char *name, int band, float bhs); // Preps the data groups for a new player
	void Get_Player(int ID, Struct_Player *tmp); // Returns the player information
	void Set_Player(int ID, Struct_Player temp); // Sets the player information
	void Grant_Exp(int ID, double ammount); // Gives the player with id (pid) 'ammount' worth of exp
	Struct_Honor Get_Honor(std::string name); // Returns the player honor information
	void Set_Honor(std::string name, Struct_Honor temp); // Sets the player honor information
	void Reset_Honor_All(); // called at the end of every map to reset veterancy stats for everyone.
	const char *GetIconBone(const char *Preset, int IconNum); // Gets the bone name for the Icon to attach
	void DropWeapon(const char *preset, Vector3 pos); // Creates a drop weapon
	void Pickup_Snd(GameObject *sender, const char *wname); // Sends the sound for the drop weapon
	void buy_item(const char *preset, GameObject *buyer, int price, int type); // Purchases an item
	int true_rand(int min, int max); // Random number
	bool bind(int pid, GameObject *tank); // Binds a vehicle to a player.
	bool unbind(int pid, std::string reason); // UnBinds a vehicle.
	int Veh_Owner(int tank_id); // Given the id of a tank, it will return 0 for no owner or the player id of the owner
	int Get_Veh_Owner(int pid); // Given the id of a owner, it will return 0 for no tank or the tank id
	std::string phrase_map(std::string map_name); // Strips the mapname of extension and the C&C
	int within_rtc(std::string name, int team);
	void Defuse_All(std::string name);
}

namespace Irc {
	void Send_Raw(const char *data); // Send raw information
	void Send_Chan(const char *chan, const char *data); // Send a channel message (chan can be name)
	void Send_Chan2(int chan, const char *data, bool system_msg = false); // Send to channel (0-both, 1-public, 2-admin)
	void Send_Notice(const char *person, const char *data); // Send a Notice (person can be channel)
	int Get_Irc_Rank(const char *name, int chan); // Get the rank of an IRC user
}

namespace RG {
	void send_rg(const char *msg);
	void send_raw(const char *msg);
	std::string encode64(const char *msg);
	std::string encode64_2(const char *msg);
}

namespace XpzBase {
	static inline bool is_base64(unsigned char c);
	std::string base64_encode(unsigned char const* bytes_to_encode, unsigned int in_len);
	std::string base64_decode(std::string const &encoded_string);
	void ipNumToString(unsigned int ipnum, char *ipstring);
	bool hostnameToIP(char *hostname, char *hostIP);
	bool is_ip(char *a, int *v);
	void send2(SOCKET sock, const char *msg);
	double pow(double base_num, int pow_of);
}

// User Commands, Id refers to player ID
namespace XpzCmds {
	void help(int type, std::string person, std::string cmd); // type: 0-irc, 1-ingame
	void rules(int type, bool is_map); // type: 0-irc, 1-ingame, is_map == true then it sends map rules
	void player(int type, int chan); // type: 0-irc, 1-ingame
	void player_info(int type, std::string sender, int id); // type: 0-irc, 1-ingame
	void game_info(int type, int chan); // type: 0-!gi, 1-!sgi
	void maps(int type, int id, int chan); // [type: 0-irc, 1-ingame] [id: 0-shownext, 1-rotation, 2-current]
	void showmods(int type, int chan); // type: 0-irc, 1-ingame
	void mine_limit(int type, int chan); // type: 0-irc, 1-ingame
	void rg_check(int type, int id, int chan); // type: 0-irc, 1-ingame - id: person to check - chan: channel OR person checking
	void buildings(int side, int chan); // Outputs buildings list to IRC
	void vehicles(int chan); // Outputs a Vehicles list to IRC
	void msg(int type, std::string sender, std::string content); // type: 0-msg. 1-amsg
	void page(std::string sender, std::string name, std::string content, int chan); // Ingame Page
	void ppage(int type, std::string sender, int id, std::string content, int chan); // type: 0-ppage, 1-apage
	void ppage(int type, int id, std::string content); // Overloaded ppage, dosn't need sender name
	void donate(GameObject *from, GameObject *to, int ammount, bool everyone = false);
	void kick(std::string kicker, std::string reason, int type, int kicked, int time = 0); // Type:0-qkick,1-kick,2-ban Time(seconds)
	void kick(std::string kicker, std::string reason, int type, const char *name, int time = 0); // Type:0-qkick,1-kick,2-ban(ip kick-3, ip ban-4) Time(seconds)
	void unban(int type, std::string kicker, std::string person); // Type:1-ip,2-name - Kicker = person unbanning - person = ip or name of person being unbanned
};

namespace XpzMod {
	std::string rank_name(int rank); // Get the name of the moderator rank by it's number
	void promote(int id, int rank, std::string flags = ""); // promote <ID> to <rank> in moderation stats
	void demote(int id); // Demotes someone to 0 rank
	void login(std::string name, std::string pass); // Checks the password and then logs the person in
}

void ObjectCreateHookBaseHook(void *data,GameObject *obj);

class Xpz_KeyHook : public ScriptImpClass {
public:
	KeyHookStruct *k;
	int hookid,is_keyhook_set;
	int AddHook(const char *Key, GameObject *obj);
	void RemoveHook();
	void Detach(GameObject *obj);
	void Destroyed(GameObject *obj);
	virtual void KeyHookFunc() = 0;
};

class Xpz_Login : public Xpz_KeyHook {
	void Created(GameObject *obj);
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
	void KeyHookFunc();
	GameObject *person;
};

class XpzElite_Tank : public ScriptImpClass {
	void Created(GameObject *obj);
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
	void Damaged(GameObject *obj,GameObject *damager,float damage);
	void Killed(GameObject *obj, GameObject *shooter);
	void Destroyed(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
	int v_owner, last_in, logos;
	bool locked;
	GameObject *lock_logo, *lock_logo2;
};

class XpzElite_Tank_Killed : public ScriptImpClass {
	void Created(GameObject *obj);
	void Damaged (GameObject *obj,GameObject *damager,float damage);
	int needed;
};

class XpzElite_Player : public ScriptImpClass {
	void Created(GameObject *obj);
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
	void Killed(GameObject *obj, GameObject *shooter);
	void Destroyed(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
	int interval;
};

class XpzElite_Player_Killed : public ScriptImpClass {
	void Created(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
	int pid;
};


class XpzElite_Player_Level : public ScriptImpClass {
	void Created(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
};

class Xpz_Drop_Weapon : public ScriptImpClass {
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
	void Created(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
};

class M00_BUILDING_EXPLODE_NO_DAMAGE_DAK : public ScriptImpClass {
	void Created(GameObject *obj);
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
	void Damaged(GameObject *obj,GameObject *damager,float damage);
	void Killed(GameObject *obj, GameObject *shooter);
	void Destroyed(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
	int interval;
};

class XpzElite_Beacon : public ScriptImpClass {
	void Created(GameObject *obj);
	void Killed(GameObject *obj, GameObject *shooter);
	void Damaged(GameObject *obj,GameObject *damager,float damage);
	void Timer_Expired(GameObject *obj, int number);
	int life;
};

class XpzElite_Timer : public ScriptImpClass {
	void Created(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
	int int_sql;
	__int64 last_sfps;
};

class XpzElite_Crates : public ScriptImpClass {
	void Created(GameObject *obj);
	void Timer_Expired(GameObject *obj, int number);
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
	int base_time, var_time, num_crates;
	bool active, loaded, loaded2; // if there is already a crate/info loaded from db
	Vector3 last_loc;
	int crate_id;
	vector<Vector3> in_crates;
};

class XpzElite_Crate : public ScriptImpClass {
	void Custom(GameObject *obj, int message, int param, GameObject *sender);
};

class m00_crate : public ScriptImpClass {
	void Created(GameObject *obj);
};

class XpzElite_C4 : public ScriptImpClass {
	void Killed(GameObject *obj, GameObject *shooter);
};