struct task {
	task();
	task(int id1, const char *parm1=NULL, const char *param2=NULL, const char *param3=NULL, double param4=0, int param5=0);
	int id;
	char parms[4096];
	char parms2[1024];
	char parms3[1024];
	double parms4;
	int parms5;
};

namespace openutils {
	class ThreadException;
	class Mutex;

	/** class Thread
	  * Represents a thread of execution
	  * in the process. To create a new Thread
	  * write an inherited class ot Thread and
	  * override the run() function
	**/
	class Thread {

	private:
		// unsigned long* to the low-level thread object
		unsigned long* m_hThread;
		int task_size;
		queue<task> task_list;
		// a name to identify the thread
		string m_strName;

	public:
		Thread();
		Thread(const char* nm);
		~Thread();
		void setName(const char* nm);
		string getName() const;
		void start();
		virtual void run();
		void sleep(long ms);
		void suspend();
		void resume();
		void stop();

		void setPriority(int p);

		bool wait(const char* m,long ms=5000);
		void release(const char* m);
		void add_task(task t_task);
		task pop_task();
		int get_tasks();

	public:
		// Thread priorities
		static const int P_ABOVE_NORMAL;
		static const int P_BELOW_NORMAL;
		static const int P_HIGHEST;
		static const int P_IDLE;
		static const int P_LOWEST;
		static const int P_NORMAL;
		static const int P_CRITICAL;
	private:				
	};// class Thread	

	/** class Mutex
	  * Represents a Mutex object to synchronize
	  * access to shaed resources.
	**/
	class Mutex2 {
	private:
		// unsigned long* to the low-level mutex object
		unsigned long* m_hMutex;
		// name to identify the mutex
		string m_strName;
	public:
		Mutex2();
		Mutex2(const char* nm);
		void create(const char* nm);
		unsigned long* getMutexHandle();
		string getName();
		void release();
		~Mutex2();
	};

	/** class ThreadException
	  * thrown by Thread and Mutex function 
	  * calls
	**/
	class ThreadException {
	private:
		string msg;
	public:
		ThreadException(const char* m);
		string getMessage() const;
	};	
} // namespace openutils

// global function called by the thread object.
// this in turn calls the overridden run()
extern "C" {
	unsigned int _ou_thread_proc(void* param);
}


				
